/*
 * Copyright (c) 1999-2001 by Peter Simons <simons@ieee.org>.
 * All rights reserved.
 */

// ISO C++ headers.
#include <fstream>

// My own libraries.
#include "../RegExp/RegExp.hh"
#include "../system-error/system-error.hh"
#include "parse-config-file.hh"

using namespace std;

inline void remove_trailing_whitespace(string& data)
    {
    if (data.empty())
        return;
    string::size_type i = data.size();
    while(i > 0 && (data[i-1] == ' ' || data[i-1] == '\t'))
        --i;
    if (i < data.size())
        data.erase(i);
    }

AbstractConfig::~AbstractConfig()
    {
    }

void parse_config_file(const char* filename, AbstractConfig& config)
    {
    // Open the config file.

    ifstream file(filename);

    if (!file)
        throw system_error(string("parse_config_file() failed to open '") + filename + "'");

    // Now we read line by line and process each one seperately.

    const RegExp    parser("^([^\t ]+)[\t ]+([^\t ].*)$", REG_EXTENDED);
    const RegExp    comment("^[\t ]*#.*$", REG_EXTENDED);
    const RegExp    empty("^[\t ]*$", REG_EXTENDED);
    vector<string>  results;
    string          line;

    for (getline(file, line); file; getline(file, line))
	{
	if (line == empty)
	    continue;
	else if (line == comment)
	    continue;
	else if (parser.submatch(line, results))
	    {
	    // A hit. Find the substrings.

	    string& keyword = results[1];
	    string& data    = results[2];
            remove_trailing_whitespace(data);

	    // Call the set_option() method in the provided class.

	    config.set_option(keyword, data);
	    }
	else
	    config.unknown_line(line);
	}
    }
