/*
 * Copyright (c) 1999-2001 by Peter Simons <simons@ieee.org>.
 * All rights reserved.
 */

// ISO C++ headers.
#include <iostream>
#include <stdexcept>

// My own libraries.
#include "parse-config-file.hh"

using namespace std;

int main(int argc, char** argv)
try
    {
    // Check the command line syntaxt.

    if (argc == 1)
        {
        std::cerr << "Usage: " << argv[0] << " configfile [configfile...]" << std::endl;
	return 1;
        }

    // This is the class we hand in to parse_config_file() in order to
    // process the found directives.

    class TestConfig : public AbstractConfig
        {
      private:
	void set_option(const string& keyword, const string& data)
            {
	    cout << "'" << keyword << "'"
		 << " = "
		 << "'" << data << "'" << endl;
            }

	void unknown_line(const string& line)
            {
	    cout << "Unknown line: '" << line << "'" << endl;
            }
        };
    TestConfig  config;

    // Now parse the files provided via the command line.

    for (int i = 1; i < argc; ++i)
        {
	try
            {
            if (i > 1)
                cout << endl;
	    cout << "Parsing file \"" << argv[i] << "\":" << endl;
	    parse_config_file(argv[i], config);
            }
	catch(runtime_error& e)
            {
	    cout << e.what() << endl;
            }
        }

    // Done.

    return 0;
    }
catch(const exception& e)
    {
    cerr << "Caught exception: " << e.what() << endl;
    return 1;
    }
catch(...)
    {
    cerr << "Caught unknown exception. Aborting." << endl;
    return 2;
    }
